// Stopperóra időmultiplex kijelzéssel
// Az időmérés alapja az 50 MHz-es órajelel leosztásával előállított
// 1 Hz-es órajel (oneHz).
// A másodperc és perc számlálás 60-nál csordul túl, az órák száma 24-nél.
// K1 gomb - stopper indítás/leállítás
// K2 gomb - stopper nullázás
 
module clock (clk_50M,key_in,digit,segment);
  input clk_50M;                            // 50 MHz órajel (pn 12)
  input [1: 0] key_in;                      // K1 és K2 nyomógombik állapota
  output reg [7: 0] digit;                  // számjegy választó regiszter
  output reg [7: 0] segment;                // szegmensvezérlő regiszter

  reg [ 3: 0] disp_dat;                     // A kiírandó számgyegy kódja
  reg [24: 0] count;                        // frekvenciaosztó számláló
  reg [23: 0] hour;                         // az idő számlálója
  reg oneHz, keyen;                         // 1 Hz jel és az állapotjelző
  reg [1: 0] dout1, dout2, dout3;           // pergésmentesítő regiszterek
  wire [1: 0] key_done;                     // Press to erase the output

    
  always @ (posedge clk_50M)                // oneHz jel generálása
    begin
      count = count + 1'b1;
      if (count == 25'd25000000)            // 0.5 s eltelt?
        begin
          count = 25'd0;                    // számláló törlése
          oneHz = ~ oneHz;                  // a másodperc jel generálása
      end
    end

    assign key_done = (dout1|dout2|dout3);  // pergésmentesített gomb állapotok

    always @ (posedge count [17]) begin     // mintavételezés 5 ms-onként
        dout1 <= key_in;
        dout2 <= dout1;
        dout3 <= dout2;
    end

    always @ (negedge key_done [0])
        keyen = ~ keyen;                    // a K1 gomb lenyomásakor átbillen

    // Időmultiplex kijelzés vezérlése 
    always @ (posedge clk_50M)              // count [17:15] kb. 1ms-onként változik
      begin
        case (count [17:15])                // a kijelzendő szám kiválasztása
            3'd0: disp_dat = hour [3: 0];   // másodpercek
            3'd1: disp_dat = hour [7: 4];   // 10 másodpercek
            3'd2: disp_dat = 4'ha;          // "-" tagolójel
            3'd3: disp_dat = hour [11: 8];  // percek
            3'd4: disp_dat = hour [15:12];  // 10 percek
            3'd5: disp_dat = 4'ha;          // "-" tagolójel
            3'd6: disp_dat = hour [19:16];  // órák
            3'd7: disp_dat = hour [23:20];  // 10 órák
        endcase
        case (count [17:15])                // számjegyválasztó jel előállítása
            3'd0: digit = 8'b11111110;      // jobb szélső számjegy
            3'd1: digit = 8'b11111101;      // 2. számjegy
            3'd2: digit = 8'b11111011;      // 3. számjegy
            3'd3: digit = 8'b11110111;      // 4. számjegy
            3'd4: digit = 8'b11101111;      // 5. számjegy
            3'd5: digit = 8'b11011111;      // 6. számjegy
            3'd6: digit = 8'b10111111;      // 7. számjegy
            3'd7: digit = 8'b01111111;      // bal szélső számjegy
        endcase
      end

    always @ (posedge clk_50M) 
	   begin
        case (disp_dat)                     // 7-szegmens dekóder
            4'h0: segment = 8'hc0;          // 0 kijelzése
            4'h1: segment = 8'hf9;          // 1 kijelzése
            4'h2: segment = 8'ha4;          // 2 kijelzése
            4'h3: segment = 8'hb0;          // 3 kijelzése
            4'h4: segment = 8'h99;          // 4 kijelzése
            4'h5: segment = 8'h92;          // 5 kijelzése
            4'h6: segment = 8'h82;          // 6 kijelzése
            4'h7: segment = 8'hf8;          // 7 kijelzése
            4'h8: segment = 8'h80;          // 8 kijelzése
            4'h9: segment = 8'h90;          // 9 kijelzése
            4'ha: segment = 8'hbf;          // - kijelzése
            default:
                segment = 8'hff;            // ne mutasd!
        endcase
        if ((count [17:15] == 3'd2) & oneHz)
            segment = 8'hff;                // a 2. elválasztójel villogtása
      end

    always @ (negedge oneHz or negedge key_done [1])            // az időszámláló kezelése
    begin
        if (! key_done [1])                                     // Ha RESET (K2) lenyomás volt
        begin
            hour = 24'h0;                                       // számláló törlése
        end
        else if (! keyen) begin
            hour [3: 0] = hour [3: 0] + 1'b1;                   // 1 másodperc hozzáadás
            if (hour [3: 0] == 4'ha) begin
                hour [3: 0] = 4'h0;
                hour [7: 4] = hour [7: 4] + 1'b1;               // 10 másodpercek léptetése
                if (hour [7: 4] == 4'h6) begin
                    hour [7: 4] = 4'h0;
                    hour [11: 8] = hour [11: 8] + 1'b1;         // túlcsordulás 60-nál
                    if (hour [11: 8] == 4'ha) begin
                        hour [11: 8] = 4'h0;
                        hour [15:12] = hour [15:12] + 1'b1;     // 10 percek léptetése
                        if (hour [15:12] == 4'h6) begin
                            hour [15:12] = 4'h0;
                            hour [19:16] = hour [19:16] + 1'b1; // túlcsordulás 60-nál
                            if (hour [19:16] == 4'ha) begin
                                hour [19:16] = 4'h0;
                                hour [23:20] = hour[23:20]+1'b1;// 10 órák léptetése
                            end
                                if (hour [23:16] == 8'h24)      // túlcsordulás 24-nél
                                    hour [23:16] = 8'h0;
                        end
                    end
                end
            end
        end
    end
    endmodule