﻿// Nyolc számjegy kijelzése időmultiplex módon
// A nyolc számjegyet felváltva jelenítjük meg
// Mindegyik helyiértéken a számjegy sorszámt jelezzük ki
 
module display (clk_50M, digit, segment);

input clk_50M;                         // 50MHz órajel (pin12)
output reg [7:0] digit;                // számjegyek vezérlése
output reg [7:0] segment;              // DP és gfedcba szegmensek vezérlése

reg [24:0] count;                      // számláló a frekvencia leosztásához
reg [3:0] data;                        // a kiírandó számjegy

always @ (posedge clk_50M)             // frekvencia leosztás
 begin
  count <= count + 1;                  // számláló léptetése
 end
 
always @ (negedge count[15])            
 begin
  data <= {1'b0, count[18:16]};        // a számjegy sorszámát íratjuk ki
  case (count[18:16])                  // a frekvenciosztó vezérli a kijelzést
       3'b000: digit <= 8'b1111_1110;  // jobb szélső számjegy aktiválása 
       3'b001: digit <= 8'b1111_1101;
       3'b010: digit <= 8'b1111_1011;
       3'b011: digit <= 8'b1111_0111;
       3'b100: digit <= 8'b1110_1111;
       3'b101: digit <= 8'b1101_1111;
       3'b110: digit <= 8'b1011_1111;
       3'b111: digit <= 8'b0111_1111;  // bal szélső számjegy aktiválása
       default:
               digit = 8'b1111_1110;
  endcase
 end 

always @ (data)
 begin
  case (data)                          // A frekvenciaosztó felső bitjei
   0: segment = 8'b11000000;           // Közös anódú szegmensek - negatív logika  
   1: segment = 8'b11111001;
   2: segment = 8'b10100100;
   3: segment = 8'b10110000;
   4: segment = 8'b10011001;
   5: segment = 8'b10010010;
   6: segment = 8'b10000010;
   7: segment = 8'b11111000;
   8: segment = 8'b10000000;
   9: segment = 8'b10010000;
  10: segment = 8'b10001000;
  11: segment = 8'b10000011;
  12: segment = 8'b11000110;
  13: segment = 8'b10100001;
  14: segment = 8'b10000110;
  15: segment = 8'b10001110;
  endcase
 end
endmodule